# Copyright 2019 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

"""
Usage:
    netq [<hostname>] show interface-stats [errors | all] [<physical-port>] [around <text-time>] [json]

Options:
    interface-stats                 : Interface statistics
    errors                          : Show TX/RX Errors
"""
from __future__ import absolute_import
from collections import OrderedDict

from netq_lib.common.enums import (
    HeaderLength,
    ConfKw
)

from netq_apps.modules import NetqModule
from netq_apps.cmd.netq import netq_show

from netq_lib.orm.redisdb.models import (
    ProcNetDevInterface
)
app = NetqModule()


def col_len(context):
    long_headers = ['Hostname', 'Interface', 'Duration', 'Last Changed']
    context.col_sizes = []
    for header in context.col_headers:
        if header in long_headers:
            context.col_sizes.append(HeaderLength.get_col_size(header))
        else:
            context.col_sizes.append(ConfKw.COL_LEN_64_BIT)


@app.route('show interface-stats')
@netq_show
def cli_show_interface_stats(cli, netq, context):
    hostname = cli.get('<hostname>')
    ifname = cli.get('<physical-port>')

    entries = ProcNetDevInterface.query.request(hostname=hostname, ifname=ifname, timestamp=netq.start_time)
    if cli.get('errors'):
        context.col_headers = ['Hostname', 'Interface', 'Duration', 'RX Bytes', 'RX Drop', 'RX Errors',
                               'TX Bytes', 'TX Drop', 'TX Errors', 'Last Changed']
        col_len(context)
        for entry in entries:
            if entry.hostname in netq.rotten_nodes or entry.rx_errs == 0 or entry.tx_errs == 0:
                continue
            out = OrderedDict()
            out['Hostname'] = entry.hostname
            out['Interface'] = entry.ifname
            out['Duration'] = entry.duration
            out['RX Bytes'] = entry.rx_bytes
            out['RX Drop'] = entry.rx_drop
            out['RX Errors'] = entry.rx_errs
            out['TX Bytes'] = entry.tx_bytes
            out['TX Drop'] = entry.tx_drop
            out['TX Errors'] = entry.tx_errs
            out['Last Changed'] = entry.timestamp
            yield out

    elif cli.get('all'):
        context.col_headers = ['Hostname', 'Interface', 'Duration', 'RX Bytes', 'RX Drop', 'RX Errors', 'RX Frame',
                               'RX Multicast', 'RX Packets', 'TX Bytes', 'TX Carrier', 'TX Colls', 'TX Drop',
                               'TX Errors', 'TX Packets', 'Last Changed']
        col_len(context)
        for entry in entries:
            if entry.hostname in netq.rotten_nodes:
                continue
            out = OrderedDict()
            out['Hostname'] = entry.hostname
            out['Interface'] = entry.ifname
            out['Duration'] = entry.duration
            out['RX Bytes'] = entry.rx_bytes
            out['RX Drop'] = entry.rx_drop
            out['RX Errors'] = entry.rx_errs
            out['RX Frame'] = entry.rx_frame
            out['RX Multicast'] = entry.rx_multicast
            out['RX Packets'] = entry.rx_packets
            out['TX Bytes'] = entry.tx_bytes
            out['TX Carrier'] = entry.tx_carrier
            out['TX Colls'] = entry.tx_colls
            out['TX Drop'] = entry.tx_drop
            out['TX Errors'] = entry.tx_errs
            out['TX Packets'] = entry.tx_packets
            out['Last Changed'] = entry.timestamp
            yield out

    else:
        context.col_headers = ['Hostname', 'Interface', 'Duration', 'RX Bytes', 'RX Drop', 'RX Errors',
                               'TX Bytes', 'TX Drop', 'TX Errors', 'Last Changed']
        col_len(context)
        for entry in entries:
            if entry.hostname in netq.rotten_nodes:
                continue
            out = OrderedDict()
            out['Hostname'] = entry.hostname
            out['Interface'] = entry.ifname
            out['Duration'] = entry.duration
            out['RX Bytes'] = entry.rx_bytes
            out['RX Drop'] = entry.rx_drop
            out['RX Errors'] = entry.rx_errs
            out['TX Bytes'] = entry.tx_bytes
            out['TX Drop'] = entry.tx_drop
            out['TX Errors'] = entry.tx_errs
            out['Last Changed'] = entry.timestamp
            yield out
